@@ -1,5 +1,7 @@
 use crate::algorithm::Printer;
+use crate::fixup::FixupContext;
 use crate::iter::IterDelimited;
+use crate::mac;
 use crate::path::PathKind;
 use crate::INDENT;
 use proc_macro2::TokenStream;
@@ -15,6 +17,7 @@ use syn::{
 impl Printer {
     pub fn item(&mut self, item: &Item) {
         match item {
+            #![cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             Item::Const(item) => self.item_const(item),
             Item::Enum(item) => self.item_enum(item),
             Item::ExternCrate(item) => self.item_extern_crate(item),
@@ -31,7 +34,6 @@ impl Printer {
             Item::Union(item) => self.item_union(item),
             Item::Use(item) => self.item_use(item),
             Item::Verbatim(item) => self.item_verbatim(item),
-            #[cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             _ => unimplemented!("unknown Item"),
         }
     }
@@ -47,7 +49,7 @@ impl Printer {
         self.ty(&item.ty);
         self.word(" = ");
         self.neverbreak();
-        self.expr(&item.expr);
+        self.expr(&item.expr, FixupContext::NONE);
         self.word(";");
         self.end();
         self.hardbreak();
@@ -91,13 +93,17 @@ impl Printer {
         self.outer_attrs(&item.attrs);
         self.cbox(INDENT);
         self.visibility(&item.vis);
-        self.signature(&item.sig);
+        self.signature(
+            &item.sig,
+            #[cfg(feature = "verbatim")]
+            &verbatim::Safety::Disallowed,
+        );
         self.where_clause_for_body(&item.sig.generics.where_clause);
         self.word("{");
         self.hardbreak_if_nonempty();
         self.inner_attrs(&item.attrs);
-        for stmt in &item.block.stmts {
-            self.stmt(stmt);
+        for stmt in item.block.stmts.iter().delimited() {
+            self.stmt(&stmt, stmt.is_last);
         }
         self.offset(-INDENT);
         self.end();
@@ -164,7 +170,7 @@ impl Printer {
 
     fn item_macro(&mut self, item: &ItemMacro) {
         self.outer_attrs(&item.attrs);
-        let semicolon = true;
+        let semicolon = mac::requires_semi(&item.mac.delimiter);
         self.mac(&item.mac, item.ident.as_ref(), semicolon);
         self.hardbreak();
     }
@@ -206,7 +212,7 @@ impl Printer {
         self.ty(&item.ty);
         self.word(" = ");
         self.neverbreak();
-        self.expr(&item.expr);
+        self.expr(&item.expr, FixupContext::NONE);
         self.word(";");
         self.end();
         self.hardbreak();
@@ -374,6 +380,7 @@ impl Printer {
 
         enum ItemVerbatim {
             Empty,
+            Ellipsis,
             ConstFlexible(FlexibleItemConst),
             FnFlexible(FlexibleItemFn),
             ImplFlexible(ImplFlexible),
@@ -447,6 +454,9 @@ impl Printer {
             fn parse(input: ParseStream) -> Result<Self> {
                 if input.is_empty() {
                     return Ok(ItemVerbatim::Empty);
+                } else if input.peek(Token![...]) {
+                    input.parse::<Token![...]>()?;
+                    return Ok(ItemVerbatim::Ellipsis);
                 }
 
                 let mut attrs = input.call(Attribute::parse_outer)?;
@@ -572,6 +582,10 @@ impl Printer {
             ItemVerbatim::Empty => {
                 self.hardbreak();
             }
+            ItemVerbatim::Ellipsis => {
+                self.word("...");
+                self.hardbreak();
+            }
             ItemVerbatim::ConstFlexible(item) => {
                 self.flexible_item_const(&item);
             }
@@ -735,7 +749,13 @@ impl Printer {
     fn use_group(&mut self, use_group: &UseGroup) {
         if use_group.items.is_empty() {
             self.word("{}");
-        } else if use_group.items.len() == 1 {
+        } else if use_group.items.len() == 1
+            && match &use_group.items[0] {
+                UseTree::Name(use_name) => use_name.ident != "self",
+                UseTree::Rename(use_rename) => use_rename.ident != "self",
+                _ => true,
+            }
+        {
             self.use_tree(&use_group.items[0]);
         } else {
             self.cbox(INDENT);
@@ -767,12 +787,12 @@ impl Printer {
 
     fn foreign_item(&mut self, foreign_item: &ForeignItem) {
         match foreign_item {
+            #![cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             ForeignItem::Fn(item) => self.foreign_item_fn(item),
             ForeignItem::Static(item) => self.foreign_item_static(item),
             ForeignItem::Type(item) => self.foreign_item_type(item),
             ForeignItem::Macro(item) => self.foreign_item_macro(item),
             ForeignItem::Verbatim(item) => self.foreign_item_verbatim(item),
-            #[cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             _ => unimplemented!("unknown ForeignItem"),
         }
     }
@@ -781,7 +801,11 @@ impl Printer {
         self.outer_attrs(&foreign_item.attrs);
         self.cbox(INDENT);
         self.visibility(&foreign_item.vis);
-        self.signature(&foreign_item.sig);
+        self.signature(
+            &foreign_item.sig,
+            #[cfg(feature = "verbatim")]
+            &verbatim::Safety::Disallowed,
+        );
         self.where_clause_semi(&foreign_item.sig.generics.where_clause);
         self.end();
         self.hardbreak();
@@ -815,7 +839,7 @@ impl Printer {
 
     fn foreign_item_macro(&mut self, foreign_item: &ForeignItemMacro) {
         self.outer_attrs(&foreign_item.attrs);
-        let semicolon = true;
+        let semicolon = mac::requires_semi(&foreign_item.mac.delimiter);
         self.mac(&foreign_item.mac, None, semicolon);
         self.hardbreak();
     }
@@ -831,20 +855,36 @@ impl Printer {
     #[cfg(feature = "verbatim")]
     fn foreign_item_verbatim(&mut self, tokens: &TokenStream) {
         use syn::parse::{Parse, ParseStream, Result};
-        use syn::{Attribute, Token, Visibility};
-        use verbatim::{FlexibleItemFn, FlexibleItemStatic, FlexibleItemType, WhereClauseLocation};
+        use syn::{Abi, Attribute, Token, Visibility};
+        use verbatim::{
+            kw, FlexibleItemFn, FlexibleItemStatic, FlexibleItemType, WhereClauseLocation,
+        };
 
         enum ForeignItemVerbatim {
             Empty,
+            Ellipsis,
             FnFlexible(FlexibleItemFn),
             StaticFlexible(FlexibleItemStatic),
             TypeFlexible(FlexibleItemType),
         }
 
+        fn peek_signature(input: ParseStream) -> bool {
+            let fork = input.fork();
+            fork.parse::<Option<Token![const]>>().is_ok()
+                && fork.parse::<Option<Token![async]>>().is_ok()
+                && ((fork.peek(kw::safe) && fork.parse::<kw::safe>().is_ok())
+                    || fork.parse::<Option<Token![unsafe]>>().is_ok())
+                && fork.parse::<Option<Abi>>().is_ok()
+                && fork.peek(Token![fn])
+        }
+
         impl Parse for ForeignItemVerbatim {
             fn parse(input: ParseStream) -> Result<Self> {
                 if input.is_empty() {
                     return Ok(ForeignItemVerbatim::Empty);
+                } else if input.peek(Token![...]) {
+                    input.parse::<Token![...]>()?;
+                    return Ok(ForeignItemVerbatim::Ellipsis);
                 }
 
                 let attrs = input.call(Attribute::parse_outer)?;
@@ -852,15 +892,13 @@ impl Printer {
                 let defaultness = false;
 
                 let lookahead = input.lookahead1();
-                if lookahead.peek(Token![const])
-                    || lookahead.peek(Token![async])
-                    || lookahead.peek(Token![unsafe])
-                    || lookahead.peek(Token![extern])
-                    || lookahead.peek(Token![fn])
-                {
+                if lookahead.peek(Token![fn]) || peek_signature(input) {
                     let flexible_item = FlexibleItemFn::parse(attrs, vis, defaultness, input)?;
                     Ok(ForeignItemVerbatim::FnFlexible(flexible_item))
-                } else if lookahead.peek(Token![static]) {
+                } else if lookahead.peek(Token![static])
+                    || ((input.peek(Token![unsafe]) || input.peek(kw::safe))
+                        && input.peek2(Token![static]))
+                {
                     let flexible_item = FlexibleItemStatic::parse(attrs, vis, input)?;
                     Ok(ForeignItemVerbatim::StaticFlexible(flexible_item))
                 } else if lookahead.peek(Token![type]) {
@@ -887,6 +925,10 @@ impl Printer {
             ForeignItemVerbatim::Empty => {
                 self.hardbreak();
             }
+            ForeignItemVerbatim::Ellipsis => {
+                self.word("...");
+                self.hardbreak();
+            }
             ForeignItemVerbatim::FnFlexible(foreign_item) => {
                 self.flexible_item_fn(&foreign_item);
             }
@@ -901,12 +943,12 @@ impl Printer {
 
     fn trait_item(&mut self, trait_item: &TraitItem) {
         match trait_item {
+            #![cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             TraitItem::Const(item) => self.trait_item_const(item),
             TraitItem::Fn(item) => self.trait_item_fn(item),
             TraitItem::Type(item) => self.trait_item_type(item),
             TraitItem::Macro(item) => self.trait_item_macro(item),
             TraitItem::Verbatim(item) => self.trait_item_verbatim(item),
-            #[cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             _ => unimplemented!("unknown TraitItem"),
         }
     }
@@ -922,7 +964,7 @@ impl Printer {
         if let Some((_eq_token, default)) = &trait_item.default {
             self.word(" = ");
             self.neverbreak();
-            self.expr(default);
+            self.expr(default, FixupContext::NONE);
         }
         self.word(";");
         self.end();
@@ -932,14 +974,18 @@ impl Printer {
     fn trait_item_fn(&mut self, trait_item: &TraitItemFn) {
         self.outer_attrs(&trait_item.attrs);
         self.cbox(INDENT);
-        self.signature(&trait_item.sig);
+        self.signature(
+            &trait_item.sig,
+            #[cfg(feature = "verbatim")]
+            &verbatim::Safety::Disallowed,
+        );
         if let Some(block) = &trait_item.default {
             self.where_clause_for_body(&trait_item.sig.generics.where_clause);
             self.word("{");
             self.hardbreak_if_nonempty();
             self.inner_attrs(&trait_item.attrs);
-            for stmt in &block.stmts {
-                self.stmt(stmt);
+            for stmt in block.stmts.iter().delimited() {
+                self.stmt(&stmt, stmt.is_last);
             }
             self.offset(-INDENT);
             self.end();
@@ -980,7 +1026,7 @@ impl Printer {
 
     fn trait_item_macro(&mut self, trait_item: &TraitItemMacro) {
         self.outer_attrs(&trait_item.attrs);
-        let semicolon = true;
+        let semicolon = mac::requires_semi(&trait_item.mac.delimiter);
         self.mac(&trait_item.mac, None, semicolon);
         self.hardbreak();
     }
@@ -996,11 +1042,13 @@ impl Printer {
     #[cfg(feature = "verbatim")]
     fn trait_item_verbatim(&mut self, tokens: &TokenStream) {
         use syn::parse::{Parse, ParseStream, Result};
-        use syn::{Attribute, Token, Visibility};
-        use verbatim::{FlexibleItemType, WhereClauseLocation};
+        use syn::{Attribute, Ident, Token, Visibility};
+        use verbatim::{FlexibleItemConst, FlexibleItemType, WhereClauseLocation};
 
         enum TraitItemVerbatim {
             Empty,
+            Ellipsis,
+            ConstFlexible(FlexibleItemConst),
             TypeFlexible(FlexibleItemType),
             PubOrDefault(PubOrDefaultTraitItem),
         }
@@ -1016,6 +1064,9 @@ impl Printer {
             fn parse(input: ParseStream) -> Result<Self> {
                 if input.is_empty() {
                     return Ok(TraitItemVerbatim::Empty);
+                } else if input.peek(Token![...]) {
+                    input.parse::<Token![...]>()?;
+                    return Ok(TraitItemVerbatim::Ellipsis);
                 }
 
                 let attrs = input.call(Attribute::parse_outer)?;
@@ -1023,7 +1074,10 @@ impl Printer {
                 let defaultness = input.parse::<Option<Token![default]>>()?.is_some();
 
                 let lookahead = input.lookahead1();
-                if lookahead.peek(Token![type]) {
+                if lookahead.peek(Token![const]) && (input.peek2(Ident) || input.peek2(Token![_])) {
+                    let flexible_item = FlexibleItemConst::parse(attrs, vis, defaultness, input)?;
+                    Ok(TraitItemVerbatim::ConstFlexible(flexible_item))
+                } else if lookahead.peek(Token![type]) {
                     let flexible_item = FlexibleItemType::parse(
                         attrs,
                         vis,
@@ -1032,7 +1086,7 @@ impl Printer {
                         WhereClauseLocation::AfterEq,
                     )?;
                     Ok(TraitItemVerbatim::TypeFlexible(flexible_item))
-                } else if (lookahead.peek(Token![const])
+                } else if (input.peek(Token![const])
                     || lookahead.peek(Token![async])
                     || lookahead.peek(Token![unsafe])
                     || lookahead.peek(Token![extern])
@@ -1060,6 +1114,13 @@ impl Printer {
             TraitItemVerbatim::Empty => {
                 self.hardbreak();
             }
+            TraitItemVerbatim::Ellipsis => {
+                self.word("...");
+                self.hardbreak();
+            }
+            TraitItemVerbatim::ConstFlexible(trait_item) => {
+                self.flexible_item_const(&trait_item);
+            }
             TraitItemVerbatim::TypeFlexible(trait_item) => {
                 self.flexible_item_type(&trait_item);
             }
@@ -1076,12 +1137,12 @@ impl Printer {
 
     fn impl_item(&mut self, impl_item: &ImplItem) {
         match impl_item {
+            #![cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             ImplItem::Const(item) => self.impl_item_const(item),
             ImplItem::Fn(item) => self.impl_item_fn(item),
             ImplItem::Type(item) => self.impl_item_type(item),
             ImplItem::Macro(item) => self.impl_item_macro(item),
             ImplItem::Verbatim(item) => self.impl_item_verbatim(item),
-            #[cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             _ => unimplemented!("unknown ImplItem"),
         }
     }
@@ -1100,7 +1161,7 @@ impl Printer {
         self.ty(&impl_item.ty);
         self.word(" = ");
         self.neverbreak();
-        self.expr(&impl_item.expr);
+        self.expr(&impl_item.expr, FixupContext::NONE);
         self.word(";");
         self.end();
         self.hardbreak();
@@ -1113,13 +1174,17 @@ impl Printer {
         if impl_item.defaultness.is_some() {
             self.word("default ");
         }
-        self.signature(&impl_item.sig);
+        self.signature(
+            &impl_item.sig,
+            #[cfg(feature = "verbatim")]
+            &verbatim::Safety::Disallowed,
+        );
         self.where_clause_for_body(&impl_item.sig.generics.where_clause);
         self.word("{");
         self.hardbreak_if_nonempty();
         self.inner_attrs(&impl_item.attrs);
-        for stmt in &impl_item.block.stmts {
-            self.stmt(stmt);
+        for stmt in impl_item.block.stmts.iter().delimited() {
+            self.stmt(&stmt, stmt.is_last);
         }
         self.offset(-INDENT);
         self.end();
@@ -1149,7 +1214,7 @@ impl Printer {
 
     fn impl_item_macro(&mut self, impl_item: &ImplItemMacro) {
         self.outer_attrs(&impl_item.attrs);
-        let semicolon = true;
+        let semicolon = mac::requires_semi(&impl_item.mac.delimiter);
         self.mac(&impl_item.mac, None, semicolon);
         self.hardbreak();
     }
@@ -1170,6 +1235,7 @@ impl Printer {
 
         enum ImplItemVerbatim {
             Empty,
+            Ellipsis,
             ConstFlexible(FlexibleItemConst),
             FnFlexible(FlexibleItemFn),
             TypeFlexible(FlexibleItemType),
@@ -1179,6 +1245,9 @@ impl Printer {
             fn parse(input: ParseStream) -> Result<Self> {
                 if input.is_empty() {
                     return Ok(ImplItemVerbatim::Empty);
+                } else if input.peek(Token![...]) {
+                    input.parse::<Token![...]>()?;
+                    return Ok(ImplItemVerbatim::Ellipsis);
                 }
 
                 let attrs = input.call(Attribute::parse_outer)?;
@@ -1221,6 +1290,10 @@ impl Printer {
             ImplItemVerbatim::Empty => {
                 self.hardbreak();
             }
+            ImplItemVerbatim::Ellipsis => {
+                self.word("...");
+                self.hardbreak();
+            }
             ImplItemVerbatim::ConstFlexible(impl_item) => {
                 self.flexible_item_const(&impl_item);
             }
@@ -1233,15 +1306,32 @@ impl Printer {
         }
     }
 
-    fn signature(&mut self, signature: &Signature) {
+    fn signature(
+        &mut self,
+        signature: &Signature,
+        #[cfg(feature = "verbatim")] safety: &verbatim::Safety,
+    ) {
         if signature.constness.is_some() {
             self.word("const ");
         }
         if signature.asyncness.is_some() {
             self.word("async ");
         }
-        if signature.unsafety.is_some() {
-            self.word("unsafe ");
+        #[cfg(feature = "verbatim")]
+        {
+            if let verbatim::Safety::Disallowed = safety {
+                if signature.unsafety.is_some() {
+                    self.word("unsafe ");
+                }
+            } else {
+                self.safety(safety);
+            }
+        }
+        #[cfg(not(feature = "verbatim"))]
+        {
+            if signature.unsafety.is_some() {
+                self.word("unsafe ");
+            }
         }
         if let Some(abi) = &signature.abi {
             self.abi(abi);
@@ -1323,9 +1413,9 @@ impl Printer {
 
     fn static_mutability(&mut self, mutability: &StaticMutability) {
         match mutability {
+            #![cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             StaticMutability::Mut(_) => self.word("mut "),
             StaticMutability::None => {}
-            #[cfg_attr(all(test, exhaustive), deny(non_exhaustive_omitted_patterns))]
             _ => unimplemented!("unknown StaticMutability"),
         }
     }
@@ -1334,27 +1424,35 @@ impl Printer {
 #[cfg(feature = "verbatim")]
 mod verbatim {
     use crate::algorithm::Printer;
+    use crate::fixup::FixupContext;
     use crate::iter::IterDelimited;
     use crate::INDENT;
     use syn::ext::IdentExt;
-    use syn::parse::{ParseStream, Result};
+    use syn::parse::{Parse, ParseStream, Result};
     use syn::{
         braced, token, Attribute, Block, Expr, Generics, Ident, Signature, StaticMutability, Stmt,
         Token, Type, TypeParamBound, Visibility, WhereClause,
     };
 
+    pub mod kw {
+        syn::custom_keyword!(safe);
+    }
+
     pub struct FlexibleItemConst {
         pub attrs: Vec<Attribute>,
         pub vis: Visibility,
         pub defaultness: bool,
         pub ident: Ident,
+        pub generics: Generics,
         pub ty: Type,
+        pub value: Option<Expr>,
     }
 
     pub struct FlexibleItemFn {
         pub attrs: Vec<Attribute>,
         pub vis: Visibility,
         pub defaultness: bool,
+        pub safety: Safety,
         pub sig: Signature,
         pub body: Option<Vec<Stmt>>,
     }
@@ -1362,6 +1460,7 @@ mod verbatim {
     pub struct FlexibleItemStatic {
         pub attrs: Vec<Attribute>,
         pub vis: Visibility,
+        pub safety: Safety,
         pub mutability: StaticMutability,
         pub ident: Ident,
         pub ty: Option<Type>,
@@ -1379,6 +1478,13 @@ mod verbatim {
         pub where_clause_after_eq: Option<WhereClause>,
     }
 
+    pub enum Safety {
+        Unsafe,
+        Safe,
+        Default,
+        Disallowed,
+    }
+
     pub enum WhereClauseLocation {
         // type Ty<T> where T: 'static = T;
         BeforeEq,
@@ -1397,8 +1503,16 @@ mod verbatim {
         ) -> Result<Self> {
             input.parse::<Token![const]>()?;
             let ident = input.call(Ident::parse_any)?;
+            let mut generics: Generics = input.parse()?;
             input.parse::<Token![:]>()?;
             let ty: Type = input.parse()?;
+            let value = if input.parse::<Option<Token![=]>>()?.is_some() {
+                let expr: Expr = input.parse()?;
+                Some(expr)
+            } else {
+                None
+            };
+            generics.where_clause = input.parse()?;
             input.parse::<Token![;]>()?;
 
             Ok(FlexibleItemConst {
@@ -1406,7 +1520,9 @@ mod verbatim {
                 vis,
                 defaultness,
                 ident,
+                generics,
                 ty,
+                value,
             })
         }
     }
@@ -1418,7 +1534,16 @@ mod verbatim {
             defaultness: bool,
             input: ParseStream,
         ) -> Result<Self> {
-            let sig: Signature = input.parse()?;
+            let constness: Option<Token![const]> = input.parse()?;
+            let asyncness: Option<Token![async]> = input.parse()?;
+            let safety: Safety = input.parse()?;
+
+            let lookahead = input.lookahead1();
+            let sig: Signature = if lookahead.peek(Token![extern]) || lookahead.peek(Token![fn]) {
+                input.parse()?
+            } else {
+                return Err(lookahead.error());
+            };
 
             let lookahead = input.lookahead1();
             let body = if lookahead.peek(Token![;]) {
@@ -1437,7 +1562,13 @@ mod verbatim {
                 attrs,
                 vis,
                 defaultness,
-                sig,
+                safety,
+                sig: Signature {
+                    constness,
+                    asyncness,
+                    unsafety: None,
+                    ..sig
+                },
                 body,
             })
         }
@@ -1445,6 +1576,7 @@ mod verbatim {
 
     impl FlexibleItemStatic {
         pub fn parse(attrs: Vec<Attribute>, vis: Visibility, input: ParseStream) -> Result<Self> {
+            let safety: Safety = input.parse()?;
             input.parse::<Token![static]>()?;
             let mutability: StaticMutability = input.parse()?;
             let ident = input.parse()?;
@@ -1474,6 +1606,7 @@ mod verbatim {
             Ok(FlexibleItemStatic {
                 attrs,
                 vis,
+                safety,
                 mutability,
                 ident,
                 ty,
@@ -1545,19 +1678,43 @@ mod verbatim {
         }
     }
 
+    impl Parse for Safety {
+        fn parse(input: ParseStream) -> Result<Self> {
+            if input.peek(Token![unsafe]) {
+                input.parse::<Token![unsafe]>()?;
+                Ok(Safety::Unsafe)
+            } else if input.peek(kw::safe) {
+                input.parse::<kw::safe>()?;
+                Ok(Safety::Safe)
+            } else {
+                Ok(Safety::Default)
+            }
+        }
+    }
+
     impl Printer {
         pub fn flexible_item_const(&mut self, item: &FlexibleItemConst) {
             self.outer_attrs(&item.attrs);
-            self.cbox(0);
+            self.cbox(INDENT);
             self.visibility(&item.vis);
             if item.defaultness {
                 self.word("default ");
             }
             self.word("const ");
             self.ident(&item.ident);
+            self.generics(&item.generics);
             self.word(": ");
+            self.cbox(-INDENT);
             self.ty(&item.ty);
-            self.word(";");
+            self.end();
+            if let Some(value) = &item.value {
+                self.word(" = ");
+                self.neverbreak();
+                self.ibox(-INDENT);
+                self.expr(value, FixupContext::NONE);
+                self.end();
+            }
+            self.where_clause_oneline_semi(&item.generics.where_clause);
             self.end();
             self.hardbreak();
         }
@@ -1569,14 +1726,14 @@ mod verbatim {
             if item.defaultness {
                 self.word("default ");
             }
-            self.signature(&item.sig);
+            self.signature(&item.sig, &item.safety);
             if let Some(body) = &item.body {
                 self.where_clause_for_body(&item.sig.generics.where_clause);
                 self.word("{");
                 self.hardbreak_if_nonempty();
                 self.inner_attrs(&item.attrs);
-                for stmt in body {
-                    self.stmt(stmt);
+                for stmt in body.iter().delimited() {
+                    self.stmt(&stmt, stmt.is_last);
                 }
                 self.offset(-INDENT);
                 self.end();
@@ -1592,6 +1749,7 @@ mod verbatim {
             self.outer_attrs(&item.attrs);
             self.cbox(0);
             self.visibility(&item.vis);
+            self.safety(&item.safety);
             self.word("static ");
             self.static_mutability(&item.mutability);
             self.ident(&item.ident);
@@ -1602,7 +1760,7 @@ mod verbatim {
             if let Some(expr) = &item.expr {
                 self.word(" = ");
                 self.neverbreak();
-                self.expr(expr);
+                self.expr(expr, FixupContext::NONE);
             }
             self.word(";");
             self.end();
@@ -1642,5 +1800,14 @@ mod verbatim {
             self.end();
             self.hardbreak();
         }
+
+        pub fn safety(&mut self, safety: &Safety) {
+            match safety {
+                Safety::Unsafe => self.word("unsafe "),
+                Safety::Safe => self.word("safe "),
+                Safety::Default => {}
+                Safety::Disallowed => unreachable!(),
+            }
+        }
     }
 }
